use nom::bytes::complete::is_not;
use nom::bytes::complete::tag;
use nom::character::complete::space0;
use nom::combinator::not;
use nom::multi::many0;
use nom::sequence::preceded;
use nom::sequence::tuple;

use super::util::get_consumed;
use super::Context;
use crate::parser::error::Res;
use crate::parser::parser_with_context::parser_with_context;
use crate::parser::util::exit_matcher_parser;
use crate::parser::util::start_of_line;
use crate::parser::Comment;

#[tracing::instrument(ret, level = "debug")]
pub fn comment<'r, 's>(context: Context<'r, 's>, input: &'s str) -> Res<&'s str, Comment<'s>> {
    let comment_line_matcher = parser_with_context!(comment_line)(context);
    let exit_matcher = parser_with_context!(exit_matcher_parser)(context);
    let (remaining, first_line) = comment_line_matcher(input)?;
    let (remaining, remaining_lines) =
        many0(preceded(not(exit_matcher), comment_line_matcher))(remaining)?;
    let source = get_consumed(input, remaining);
    Ok((remaining, Comment { source }))
}

#[tracing::instrument(ret, level = "debug")]
fn comment_line<'r, 's>(context: Context<'r, 's>, input: &'s str) -> Res<&'s str, &'s str> {
    start_of_line(context, input)?;
    let (remaining, _indent) = space0(input)?;
    let (remaining, (_hash, _leading_whitespace, content)) =
        tuple((tag("#"), space0, is_not("\r\n")))(remaining)?;
    let source = get_consumed(input, remaining);
    Ok((remaining, source))
}
